#!/usr/bin/env python3
"""
Vulnerability Scanner
Network vulnerability assessment and security testing tool.
Parameters:
    - target: Target IP or hostname
    - ports: Port range to scan
    - modules: List of vulnerability modules to run
Requires: nmap-python, paramiko, requests packages and admin/root privileges
"""
import nmap
import paramiko
import requests
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
import socket
import ssl
import re
from concurrent.futures import ThreadPoolExecutor
import xml.etree.ElementTree as ET

class VulnerabilityScanner:
    def __init__(self, target, ports=None, modules=None, threads=10):
        self.target = target
        self.ports = ports or "1-1000"
        self.modules = modules or ['ports', 'services', 'web', 'ssh']
        self.threads = threads
        self.results_queue = queue.Queue()
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def scan_ports(self):
        """Perform port scan using nmap"""
        try:
            nm = nmap.PortScanner()
            self.logger.info(f"Starting port scan of {self.target}")
            
            # Run nmap scan
            nm.scan(self.target, self.ports, arguments='-sV -sS -O --script=vuln')
            
            for host in nm.all_hosts():
                host_result = {
                    'host': host,
                    'state': nm[host].state(),
                    'ports': []
                }

                # Get OS information
                if 'osmatch' in nm[host]:
                    host_result['os'] = [
                        {
                            'name': match['name'],
                            'accuracy': match['accuracy']
                        } for match in nm[host]['osmatch']
                    ]

                # Get port information
                for proto in nm[host].all_protocols():
                    ports = nm[host][proto].keys()
                    for port in ports:
                        port_info = nm[host][proto][port]
                        port_result = {
                            'port': port,
                            'state': port_info['state'],
                            'service': port_info['name'],
                            'product': port_info.get('product', ''),
                            'version': port_info.get('version', ''),
                            'vulnerabilities': []
                        }

                        # Parse script output for vulnerabilities
                        if 'script' in port_info:
                            for script_name, output in port_info['script'].items():
                                if 'VULNERABLE' in output:
                                    port_result['vulnerabilities'].append({
                                        'type': script_name,
                                        'details': output
                                    })

                        host_result['ports'].append(port_result)

                self.results_queue.put({
                    'module': 'ports',
                    'findings': host_result
                })

        except Exception as e:
            self.logger.error(f"Error in port scan: {str(e)}")

    def scan_web_vulnerabilities(self, port=80):
        """Scan for web application vulnerabilities"""
        try:
            url = f"http://{self.target}:{port}"
            self.logger.info(f"Scanning web vulnerabilities at {url}")
            
            vulnerabilities = []
            
            # Test for common vulnerabilities
            tests = [
                {
                    'name': 'SQL Injection',
                    'payload': "'OR '1'='1",
                    'pattern': r'SQL syntax|mysql_fetch|ORA-[0-9]|SQL error'
                },
                {
                    'name': 'XSS',
                    'payload': '<script>alert(1)</script>',
                    'pattern': r'<script>alert\(1\)</script>'
                },
                {
                    'name': 'Directory Traversal',
                    'payload': '../../../etc/passwd',
                    'pattern': r'root:|/bin/bash'
                }
            ]
            
            for test in tests:
                try:
                    response = requests.get(
                        f"{url}?q={test['payload']}",
                        timeout=5,
                        verify=False
                    )
                    if re.search(test['pattern'], response.text):
                        vulnerabilities.append({
                            'type': test['name'],
                            'url': url,
                            'details': 'Potential vulnerability detected'
                        })
                except:
                    continue

            # Check security headers
            try:
                response = requests.get(url, timeout=5, verify=False)
                headers = response.headers
                
                security_headers = {
                    'Strict-Transport-Security': 'Missing HSTS',
                    'Content-Security-Policy': 'Missing CSP',
                    'X-Frame-Options': 'Missing X-Frame-Options',
                    'X-Content-Type-Options': 'Missing X-Content-Type-Options',
                    'X-XSS-Protection': 'Missing XSS Protection'
                }
                
                for header, message in security_headers.items():
                    if header not in headers:
                        vulnerabilities.append({
                            'type': 'Missing Security Header',
                            'header': header,
                            'details': message
                        })
            except:
                pass

            # Check SSL/TLS configuration
            try:
                context = ssl.create_default_context()
                with socket.create_connection((self.target, 443)) as sock:
                    with context.wrap_socket(sock,
                                          server_hostname=self.target) as ssock:
                        cert = ssock.getpeercert()
                        
                        # Check certificate expiration
                        not_after = datetime.strptime(
                            cert['notAfter'],
                            '%b %d %H:%M:%S %Y %Z'
                        )
                        if not_after < datetime.now():
                            vulnerabilities.append({
                                'type': 'SSL Certificate',
                                'details': 'Certificate has expired'
                            })
            except:
                vulnerabilities.append({
                    'type': 'SSL/TLS',
                    'details': 'Unable to establish secure connection'
                })

            self.results_queue.put({
                'module': 'web',
                'findings': {
                    'url': url,
                    'vulnerabilities': vulnerabilities
                }
            })

        except Exception as e:
            self.logger.error(f"Error in web vulnerability scan: {str(e)}")

    def scan_ssh_vulnerabilities(self, port=22):
        """Scan for SSH vulnerabilities"""
        try:
            self.logger.info(f"Scanning SSH vulnerabilities on port {port}")
            vulnerabilities = []
            
            # Test SSH version
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            
            try:
                transport = paramiko.Transport((self.target, port))
                server_version = transport.get_banner().decode()
                
                # Check for old SSH versions
                if 'SSH-1' in server_version:
                    vulnerabilities.append({
                        'type': 'SSH Version',
                        'details': 'Using obsolete SSH-1 protocol'
                    })
                
                # Check supported algorithms
                kex = transport.get_security_options()
                
                # Check for weak ciphers
                weak_ciphers = ['aes128-cbc', 'aes192-cbc', 'aes256-cbc',
                              '3des-cbc', 'blowfish-cbc', 'cast128-cbc']
                for cipher in weak_ciphers:
                    if cipher in kex.ciphers:
                        vulnerabilities.append({
                            'type': 'Weak Cipher',
                            'details': f'Supports weak cipher: {cipher}'
                        })
                
                # Check for weak MAC algorithms
                weak_macs = ['hmac-md5', 'hmac-md5-96', 'hmac-sha1-96']
                for mac in weak_macs:
                    if mac in kex.macs:
                        vulnerabilities.append({
                            'type': 'Weak MAC',
                            'details': f'Supports weak MAC: {mac}'
                        })
                
            except Exception as e:
                vulnerabilities.append({
                    'type': 'SSH Connection',
                    'details': f'Error: {str(e)}'
                })
            finally:
                try:
                    transport.close()
                except:
                    pass

            self.results_queue.put({
                'module': 'ssh',
                'findings': {
                    'port': port,
                    'vulnerabilities': vulnerabilities
                }
            })

        except Exception as e:
            self.logger.error(f"Error in SSH vulnerability scan: {str(e)}")

    def save_results(self, output_file):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        scan_results = {
            'timestamp': datetime.now().isoformat(),
            'target': self.target,
            'ports': self.ports,
            'modules': self.modules,
            'findings': results
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(scan_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        self.logger.info("\nVulnerability Scan Summary:")
        self.logger.info(f"Target: {self.target}")
        
        total_vulns = 0
        vuln_types = {}
        
        for result in results:
            module = result['module']
            findings = result['findings']
            
            if module == 'ports':
                open_ports = len([p for p in findings['ports']
                                if p['state'] == 'open'])
                self.logger.info(f"\nOpen Ports: {open_ports}")
                
                for port in findings['ports']:
                    if port['vulnerabilities']:
                        total_vulns += len(port['vulnerabilities'])
                        for vuln in port['vulnerabilities']:
                            vuln_type = vuln['type']
                            vuln_types[vuln_type] = vuln_types.get(vuln_type, 0) + 1
            
            elif module == 'web':
                web_vulns = findings['vulnerabilities']
                total_vulns += len(web_vulns)
                for vuln in web_vulns:
                    vuln_type = vuln['type']
                    vuln_types[vuln_type] = vuln_types.get(vuln_type, 0) + 1
            
            elif module == 'ssh':
                ssh_vulns = findings['vulnerabilities']
                total_vulns += len(ssh_vulns)
                for vuln in ssh_vulns:
                    vuln_type = vuln['type']
                    vuln_types[vuln_type] = vuln_types.get(vuln_type, 0) + 1

        self.logger.info(f"\nTotal Vulnerabilities Found: {total_vulns}")
        if vuln_types:
            self.logger.info("\nVulnerability Types:")
            for vuln_type, count in vuln_types.items():
                self.logger.info(f"{vuln_type}: {count}")

    def start_scan(self, output_file=None):
        self.logger.info(f"Starting vulnerability scan of {self.target}")
        
        if 'ports' in self.modules:
            self.scan_ports()
        
        if 'web' in self.modules:
            self.scan_web_vulnerabilities()
        
        if 'ssh' in self.modules:
            self.scan_ssh_vulnerabilities()
        
        if output_file:
            self.save_results(output_file)
        
        self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Vulnerability Scanner")
    parser.add_argument("target", help="Target IP or hostname")
    parser.add_argument("--ports", default="1-1000",
                      help="Port range to scan (default: 1-1000)")
    parser.add_argument("--modules", nargs='+',
                      default=['ports', 'services', 'web', 'ssh'],
                      help="Vulnerability modules to run")
    parser.add_argument("--threads", type=int, default=10,
                      help="Number of concurrent threads")
    parser.add_argument("--output", help="Output file for scan results")
    args = parser.parse_args()

    scanner = VulnerabilityScanner(args.target, args.ports,
                                 args.modules, args.threads)
    scanner.start_scan(args.output)
